import numpy
import pandas as pd
import matplotlib.pyplot as plt

s = input("Input Excel data file (*.xlsx) = ")
xlsx_fl = pd.ExcelFile(s)
data_xlsx = pd.read_excel(xlsx_fl,  header = None)
data = data_xlsx.values

s_out = input("Output text data file (*.txt) = ")
fout = open(s_out, "w")
fout.write("Input data file = " + s + "\n")
data = data.T
n_vars = 0
var_names = []
X = []
for v in data:
    if v[1] == 1:
        var_names.append(v[0])
        X.append(list(v[2:]))
        n_vars += 1        

n_data = len(X[0])
print('n_data =', n_data)
print('n_vars =', n_vars)
print('var_names = :', var_names)
for i, nm in enumerate( var_names):
    print(f'{nm:<.10s} ==> X[{i+1}]')
print()
fout.write('\nn_data ={}\n'.format(n_data))
fout.write('n_vars = {}\n'.format(n_vars))
fout.write('var_names: {}\n'.format(var_names))
for i, nm in enumerate( var_names):
    fout.write(f'{nm:<.10s} ==> X[{i+1}]\n')
fout.write('\n')

X = numpy.array(X).T
RawScore = numpy.sum(X, axis=1)
meanX = numpy.mean(X, axis=0)
X = X - meanX                                            #    Centering of X
#
#       SVD of the centered X
#
U, Sv, Vh = numpy.linalg.svd(X, full_matrices=False)

#
#       Scree plot of sigular values
#
plt.plot(numpy.arange(1, len(Sv)+1), Sv)
plt.title('Scree Plot', fontsize= 16)
plt.ylabel('Singular value', fontsize=14)
plt.tight_layout()
fnm_scree = s_out[:-4] + '_Scree.png'
plt.savefig(fnm_scree)
plt.show()

if numpy.sum(Vh[0]) < 0.0:
    U *= -1.0
    Vh *= -1.0
S = numpy.cov(X, rowvar=False)
A = Vh[0] * Sv[0] / (n_data**0.5)     #   Vector of Coefficients, i.e. Lambda's

for i in range(n_vars):
    print('Lambda[{0}]**2 / Var(X[{0}]) = {1:.3f}'.format(
            i+1, A[i]**2 / S[i][i]))
    fout.write('Lambda[{0}]**2 / Var(X[{0}]) = {1:.3f}\n'.format(
            i+1, A[i]**2 / S[i][i]))
print()
fout.write('\n')
for i in range(n_vars):
    print('{0}: Lambda[{1}] = {2:.3f}'.format(
            var_names[i], i+1, A[i]))
    fout.write('{0}: Lambda[{1}] = {2:.3f}\n'.format(
            var_names[i], i+1, A[i]))

#
#        Calculation of GFI (Goodness of Fit Index)
#
sum_num = 0.0
sum_den = 0.0
for i in range(n_vars):
    for j in range(n_vars):
        if i == j:
            sum_den += S[i][i] ** 2
        else:
            sum_den += S[i][j] ** 2
            sum_num += (S[i][j] - A[i] * A[j]) ** 2
GFI = 1.0 - sum_num / sum_den
print('\nGFI = {0:.3f}'.format(GFI))
fout.write('\nGFI = {0:.3f}\n'.format(GFI))
#
#         Calculation of omega
#       
sx2 = numpy.var(X.sum(axis=1))
omega = (A.sum() ** 2) / sx2
print('\nω = {0:.3f}'.format(omega))
fout.write('\nω = {0:.3f}\n'.format(omega))

f = U[:,0] * (n_data**0.5)
f = numpy.reshape(f, (n_data,1))
A = numpy.reshape(A, (1, len(A)))
ModelScore = numpy.sum(f @ A, axis=1) + numpy.sum(meanX)
#
#       Scatter plot of (model prediction, raw score)
#
plt.scatter(ModelScore, RawScore)
plt.xlabel('Model Score', fontsize=14)
plt.ylabel('Raw Score', fontsize=14)
plt.plot([numpy.min(ModelScore), numpy.max(ModelScore)],[numpy.min(ModelScore), numpy.max(ModelScore)],
        label = '$Y=X$', color='k')
plt.title(f'$\\omega={omega:.3f}$', fontsize=16)
plt.legend(fontsize=14)
plt.tight_layout()
fnm_scatter = s_out[:-4] + 'Scatter.png'
plt.savefig(fnm_scatter)
plt.show()

fout.close()
print()
print(fnm_scree + ' was saved.')
print(fnm_scatter + ' was saved.')
print(s_out + " was saved.\n")
